/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

#import "BDAssignment.h"
#import "BDRuleContext.h"


@implementation BDAssignment

#pragma mark -
#pragma mark class methods

+ (id)assignmentWithKeyPath:(NSString *)keyPath
                      value:(id)value
{
    return [[self alloc] initWithKeyPath:keyPath value:value];
}

#pragma mark -
#pragma mark init methods

- (id)initWithKeyPath:(NSString *)inKeyPath value:(id)inValue
{
    if (self = [super init])
    {
        self.keyPath = inKeyPath;
        self.value = inValue;
    }

    return self;
}

#pragma -
#pragma mark dealloc
- (void)dealloc
{
    self.keyPath = nil;
    self.value = nil;
}

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods
- (void)encodeWithCoder:(NSCoder *)coder
{
	if([coder allowsKeyedCoding])
	{
		[coder encodeObject:self.keyPath forKey:@"keyPath"];
		[coder encodeObject:self.value forKey:@"value"];
	}
	else
	{
        [coder encodeObject:self.keyPath];
        [coder encodeObject:self.value];
	}
    
}

- (id)initWithCoder:(NSCoder *)coder
{
    if([coder allowsKeyedCoding])
    {
        self.keyPath = [coder decodeObjectForKey:@"keyPath"];
        self.value   = [coder decodeObjectForKey:@"value"];
    }
    else
    {
        self.keyPath = [coder decodeObject];
        self.value   = [coder decodeObject];
    }
    
    return self;
}

#pragma mark -
#pragma mark BDPropertyListEncoding methods
- (id)initWithPropertyList:(NSDictionary *)propertyList
                     owner:(id)owner
{
    int version;
    NSString *theKeyPath = nil;
    id theValue = nil;
    
    version = [[propertyList valueForKey:@"version"] intValue];
    
    if (version >= 1) {
        theKeyPath = [propertyList valueForKey:@"keyPath"];
        
        theValue = [propertyList valueForKey:@"value"];
    }
    
    return [self initWithKeyPath:theKeyPath
                           value:theValue];
}

- (void)awakeWithPropertyList:(NSDictionary *)propertyList
{
    // Do nothing.
}

- (void)encodeIntoPropertyList:(NSMutableDictionary *)propertyList
{
    // Encode a version number
    
    [propertyList setValue:[NSNumber numberWithInt:1]
                    forKey:@"version"];
    
    // Encode the key path
    
    [propertyList setValue:self.keyPath
                    forKey:@"keyPath"];
    
    // Encode the value as a string.
    
    [propertyList setValue:[self.value description]
                    forKey:@"value"];
}

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

@synthesize keyPath;
@synthesize value;

#pragma mark -
#pragma mark Utility methods

- (id)fireInContext:(BDRuleContext *)context
{
    return [self value];
}

- (NSString *)description
{
    NSString *s;

    if ([[self value] isKindOfClass:[NSString class]])
    {
        s = [NSString stringWithFormat:@"%@ = \"%@\"", self.keyPath, self.value];
    }
    else
    {
        s = [NSString stringWithFormat:@"%@ = %@", self.keyPath, [self.value description]];
    }

    return s;
}

@end
