//
//  TestBDExpressionAssignment.m
//  TestBDExpressionAssignment
//
//  Created by narita on 2015/03/17.
//  Copyright (c) 2015年 narita. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <XCTest/XCTest.h>
#import "BDExpressionAssignment.h"

@interface TestBDExpressionAssignment : XCTestCase

@end

@implementation TestBDExpressionAssignment

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (void)testAssignmentEncodeIntoPropertyList
{
    BDExpressionAssignment *assignment;
    NSMutableDictionary *plist;
    
    assignment = [BDExpressionAssignment assignmentWithKeyPath:@"foo.bar"
                                                         value:@"5 + 6 + foo"];
    
    plist = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [assignment encodeIntoPropertyList:plist];
    
    XCTAssertTrue([plist count] == 3, @"");
    XCTAssertTrue([[plist valueForKey:@"version"] intValue] == 1, @"");
    XCTAssertEqualObjects([plist valueForKey:@"keyPath"], @"foo.bar", @"");
    XCTAssertEqualObjects([plist valueForKey:@"value"], @"5 + 6 + foo", @"");
}

- (void)testAssignmentInitWithPropertyList
{
    BDExpressionAssignment *assignment;
    NSMutableDictionary *plist;
    
    plist = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [plist setValue:[NSNumber numberWithInt:1]
             forKey:@"version"];
    
    [plist setValue:@"foo.bar"
             forKey:@"keyPath"];
    
    [plist setValue:@"5 + 6 + foo"
             forKey:@"value"];
    
    assignment = [[BDExpressionAssignment alloc] initWithPropertyList:plist
                                                      owner:nil];
    
    [assignment awakeWithPropertyList:plist];
    
    XCTAssertEqualObjects([assignment keyPath], @"foo.bar", @"");
    XCTAssertEqualObjects([assignment value], @"5 + 6 + foo", @"");
}

- (void)testRuleEncodeIntoPropertyList
{
    BDRule *rule;
    NSMutableDictionary *plist;
    
    rule = [BDRule ruleWithLhs:[NSPredicate predicateWithFormat:@"stock.symbol == 'aapl'"]
                           rhs:[BDExpressionAssignment assignmentWithKeyPath:@"foo.bar"
                                                                       value:@"5 + 6 + foo"]
                      priority:75];
    
    plist = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [rule encodeIntoPropertyList:plist];
    
    XCTAssertTrue([plist count] == 5, @"");
    XCTAssertTrue([[plist valueForKey:@"version"] intValue] == 1, @"");
    XCTAssertEqualObjects([plist valueForKey:@"lhs"], @"stock.symbol == \"aapl\"", @"");
    XCTAssertEqualObjects([plist valueForKey:@"assignmentClass"], @"BDExpressionAssignment", @"");
    
    XCTAssertTrue([[plist valueForKeyPath:@"rhs.version"] intValue] == 1, @"");
    XCTAssertEqualObjects([plist valueForKeyPath:@"rhs.keyPath"], @"foo.bar", @"");
    XCTAssertEqualObjects([plist valueForKeyPath:@"rhs.value"], @"5 + 6 + foo", @"");
    
    XCTAssertTrue([[plist valueForKey:@"priority"] intValue] == 75, @"");
}

- (void)testRuleInitWithPropertyList
{
    NSMutableDictionary *plist;
    BDRule *rule;
    
    plist = [NSMutableDictionary dictionaryWithCapacity:0];
    
    [plist setValue:[NSNumber numberWithInt:1]
             forKey:@"version"];
    
    [plist setValue:@"stock.symbol = 'msft'"
             forKey:@"lhs"];
    
    [plist setValue:@"BDExpressionAssignment"
             forKey:@"assignmentClass"];
    
    [plist setValue:[NSMutableDictionary dictionaryWithCapacity:0]
             forKey:@"rhs"];
    
    [plist setValue:[NSNumber numberWithInt:1]
         forKeyPath:@"rhs.version"];
    
    [plist setValue:@"actions.priceChange"
         forKeyPath:@"rhs.keyPath"];
    
    [plist setValue:@"5 + 6 + foo"
         forKeyPath:@"rhs.value"];
    
    [plist setValue:[NSNumber numberWithInt:95]
             forKey:@"priority"];
    
    rule = [[BDRule alloc] initWithPropertyList:plist
                                          owner:nil];
    
    [rule awakeWithPropertyList:plist];
    
    XCTAssertEqualObjects([[rule lhs] description], @"stock.symbol == \"msft\"", @"");
    
    XCTAssertEqualObjects(NSStringFromClass([[rule rhs] class]), @"BDExpressionAssignment", @"");
    XCTAssertEqualObjects([[rule rhs] keyPath], @"actions.priceChange", @"");
    XCTAssertEqualObjects([[rule rhs] value], @"5 + 6 + foo", @"");
    
    XCTAssertTrue([rule priority] == 95, @"");
}

- (void)testFireInContext
{
    BDExpressionAssignment *assignment;
    
    assignment = [BDExpressionAssignment assignmentWithKeyPath:@"foo"
                                                         value:@"5 + 6"];
    
    id theResult = [assignment fireInContext:nil];
    
    XCTAssertEqualObjects(theResult, @11, @"");

}

@end
